@@ -28,12 +28,12 @@ ACCEPTED_BOOKMARK_FILTERS = ['archive', 'favorite', 'domain', 'added_since'
 
 
 class BaseClient(object):
-    """A base class for Readability clients.
-
     """
-
+    A base class for Readability clients.
+    """
     def _create_response(self, response, content):
-        """Modify the httplib2.Repsonse object to return.
+        """
+        Modify the httplib2.Repsonse object to return.
 
         Add two attributes to it:
 
@@ -69,15 +69,15 @@ class BaseClient(object):
 
 
 class ReaderClient(BaseClient):
-    """Client for interacting with the Readability Reader API.
+    """
+    Client for interacting with the Readability Reader API.
 
     Docs can be found at `http://www.readability.com/developers/api/reader`.
-
     """
-
     def __init__(self, consumer_key, consumer_secret, token_key, token_secret,
         base_url_template=DEFAULT_READER_URL_TEMPLATE):
-        """Initialize the ReadeClient.
+        """
+        Initialize the ReadeClient.
 
         :param consumer_key: Reader API key
         :param consumer_secret: Reader API secret
@@ -87,30 +87,28 @@ class ReaderClient(BaseClient):
             which requests will be sent. This shouldn't need to be passed as the
             main purpose for it is testing environments that the user probably
             doesn't have access to (staging, local dev, etc).
-
         """
-
         self.base_url_template = base_url_template
         self.token = oauth2.Token(token_key, token_secret)
         self.consumer = oauth2.Consumer(consumer_key, consumer_secret)
         self.oauth_client = oauth2.Client(self.consumer, self.token)
 
     def get(self, url):
-        """Make a HTTP GET request to the Reader API.
+        """
+        Make a HTTP GET request to the Reader API.
 
         :param url: url to which to make a GET request.
-
         """
         logger.debug('Making GET request to %s', url)
         return self._create_response(
             *self.oauth_client.request(url, method='GET'))
 
     def post(self, url, post_params=None):
-        """Make a HTTP POST request ot the Reader API.
+        """
+        Make a HTTP POST request to the Reader API.
 
         :param url: url to which to make a POST request.
         :param post_params: parameters to be sent in the request's body.
-
         """
         params = urllib.urlencode(post_params)
         logger.debug('Making POST request to %s with body %s', url, params)
@@ -118,23 +116,23 @@ class ReaderClient(BaseClient):
             *self.oauth_client.request(url, method='POST', body=params))
 
     def delete(self, url):
-        """Make a HTTP DELETE request ot the Readability API.
+        """
+        Make a HTTP DELETE request to the Readability API.
 
         :param url: The url to which to send a DELETE request.
-
         """
         logger.debug('Making DELETE request to %s', url)
         return self._create_response(
             *self.oauth_client.request(url, method='DELETE'))
 
     def _generate_url(self, resource, query_params=None):
-        """Generate a Readability URL to the given resource.
+        """
+        Generate a Readability URL to the given resource.
 
         :param resource: the path to the resource that the request should
             go to.
         :param query_params (optional): a dict of query params that should
             be added to the url.
-
         """
         if query_params:
             resource = '{0}?{1}'.format(
@@ -143,16 +141,17 @@ class ReaderClient(BaseClient):
         return self.base_url_template.format(resource)
 
     def get_article(self, article_id):
-        """Get a single article represented by `article_id`.
+        """
+        Get a single article represented by `article_id`.
 
         :param article_id: ID of the article to retrieve.
-
         """
         url = self._generate_url('articles/{0}'.format(article_id))
         return self.get(url)
 
     def get_bookmarks(self, **filters):
-        """Get Bookmarks for the current user.
+        """
+        Get Bookmarks for the current user.
 
         Filters:
 
@@ -171,114 +170,165 @@ class ReaderClient(BaseClient):
         :param per_page: How many results to return per page. Default is 20, max is 50.
         :param only_deleted: Return only bookmarks that this user has deleted.
         :param tags: Comma separated string of tags to filter bookmarks.
-
         """
         filter_dict = filter_args_to_dict(filters, ACCEPTED_BOOKMARK_FILTERS)
         url = self._generate_url('bookmarks', query_params=filter_dict)
         return self.get(url)
 
     def get_bookmark(self, bookmark_id):
-        """Get a single bookmark represented by `bookmark_id`.
+        """
+        Get a single bookmark represented by `bookmark_id`.
 
         The requested bookmark must belong to the current user.
 
         :param bookmark_id: ID of the bookmark to retrieve.
-
         """
         url = self._generate_url('bookmarks/{0}'.format(bookmark_id))
         return self.get(url)
 
     def add_bookmark(self, url, favorite=False, archive=False):
-        """Adds given bookmark.
-
         """
-
+        Adds given bookmark.
+        """
         rdb_url = self._generate_url('bookmarks')
         params = dict(url=url, favorite=int(favorite), archive=int(archive))
         return self.post(rdb_url, params)
 
+    def update_bookmark(self, bookmark_id, favorite=None, archive=None, read_percent=None):
+        """
+        Updates given bookmark. The requested bookmark must belong to the
+        current user.
+
+        :param bookmark_id: ID of the bookmark to update.
+        :param favorite (optional): Whether this article is favorited or not.
+        :param archive (optional): Whether this article is archived or not.
+        :param read_percent (optional): The read progress made in this article,
+            where 1.0 means the bottom and 0.0 means the very top.
+        """
+        rdb_url = self._generate_url('bookmarks/{0}'.format(bookmark_id))
+        params = {}
+        if favorite is not None:
+            params['favorite'] = 1 if favorite == True else 0
+        if archive is not None:
+            params['archive'] = 1 if archive == True else 0
+        if read_percent is not None:
+            try:
+                params['read_percent'] = float(read_percent)
+            except ValueError:
+                pass
+        return self.post(rdb_url, params)
+
+    def favorite_bookmark(self, bookmark_id):
+        """
+        Favorites given bookmark. The requested bookmark must belong to the
+        current user.
+
+        :param bookmark_id: ID of the bookmark to favorite.
+        """
+        return self.update_bookmark(bookmark_id, favorite=True)
+
+    def archive_bookmark(self, bookmark_id):
+        """
+        Archives given bookmark. The requested bookmark must belong to the
+        current user.
+
+        :param bookmark_id: ID of the bookmark to archive.
+        """
+        return self.update_bookmark(bookmark_id, archive=True)
+
+    def set_read_percent_of_bookmark(self, bookmark_id, read_percent):
+        """
+        Set read progress of given bookmark. The requested bookmark must
+        belong to the current user.
+
+        :param bookmark_id: ID of the bookmark to update.
+        :param read_percent: The read progress made in this article,
+          where 1.0 means the bottom and 0.0 means the very top.
+        """
+        return self.update_bookmark(bookmark_id, read_percent=read_percent)
+
     def delete_bookmark(self, bookmark_id):
-        """Delete a single bookmark represented by `bookmark_id`.
+        """
+        Delete a single bookmark represented by `bookmark_id`.
 
         The requested bookmark must belong to the current user.
 
         :param bookmark_id: ID of the bookmark to delete.
-
         """
         url = self._generate_url('bookmarks/{0}'.format(bookmark_id))
         return self.delete(url)
 
     def get_bookmark_tags(self, bookmark_id):
-        """Retrieve tags that have been applied to a bookmark.
+        """
+        Retrieve tags that have been applied to a bookmark.
 
         The requested bookmark must belong to the current user.
 
         :param bookmark_id: ID of the bookmark to delete.
-
         """
         url = self._generate_url('bookmarks/{0}/tags'.format(bookmark_id))
         return self.get(url)
 
     def add_tags_to_bookmark(self, bookmark_id, tags):
-        """Add tags to to a bookmark.
+        """
+        Add tags to to a bookmark.
 
         The identified bookmark must belong to the current user.
 
         :param bookmark_id: ID of the bookmark to delete.
         :param tags: Comma separated tags to be applied.
-
         """
         url = self._generate_url('bookmarks/{0}/tags'.format(bookmark_id))
         params = dict(tags=tags)
         return self.post(url, params)
 
     def delete_tag_from_bookmark(self, bookmark_id, tag_id):
-        """Remove a single tag from a bookmark.
+        """
+        Remove a single tag from a bookmark.
 
         The identified bookmark must belong to the current user.
 
         :param bookmark_id: ID of the bookmark to delete.
-
         """
         url = self._generate_url('bookmarks/{0}/tags/{1}'.format(
             bookmark_id, tag_id))
         return self.delete(url)
 
     def get_tag(self, tag_id):
-        """Get a single tag represented by `tag_id`.
+        """
+        Get a single tag represented by `tag_id`.
 
         The requested tag must belong to the current user.
 
         :param tag_id: ID fo the tag to retrieve.
-
         """
         url = self._generate_url('tags/{0}'.format(tag_id))
         return self.get(url)
 
     def get_tags(self):
-        """Get all tags belonging to the current user.
-
+        """
+        Get all tags belonging to the current user.
         """
         url = self._generate_url('tags')
         return self.get(url)
 
     def get_user(self):
-        """Retrives the current user.
-
+        """
+        Retrives the current user.
         """
         url = self._generate_url('users/_current')
         return self.get(url)
 
 
 class ParserClient(BaseClient):
-    """Client for interacting with the Readability Parser API.
+    """
+    Client for interacting with the Readability Parser API.
 
     Docs can be found at `http://www.readability.com/developers/api/parser`.
-
     """
-
     def __init__(self, token, base_url_template=DEFAULT_PARSER_URL_TEMPLATE):
-        """Initialize client.
+        """
+        Initialize client.
 
         :param token: parser API token.
         :param base_url_template (optional): Template used to build URL to
@@ -292,33 +342,32 @@ class ParserClient(BaseClient):
         self.token = token
         self.base_url_template = base_url_template
 
-
     def get(self, url):
-        """Make an HTTP GET request to the Parser API.
+        """
+        Make an HTTP GET request to the Parser API.
 
         :param url: url to which to make the request
-
         """
         logger.debug('Making GET request to %s', url)
         http = httplib2.Http()
         return self._create_response(*http.request(url, 'GET'))
 
     def head(self, url):
-        """Make an HTTP HEAD request to the Parser API.
+        """
+        Make an HTTP HEAD request to the Parser API.
 
         :param url: url to which to make the request
-
         """
         logger.debug('Making HEAD request to %s', url)
         http = httplib2.Http()
         return self._create_response(*http.request(url, 'HEAD'))
 
     def post(self, url, post_params=None):
-        """Make an HTTP POST request to the Parser API.
+        """
+        Make an HTTP POST request to the Parser API.
 
         :param url: url to which to make the request
         :param post_params: POST data to send along. Expected to be a dict.
-
         """
         post_params['token'] = self.token
         params = urllib.urlencode(post_params)
@@ -327,12 +376,12 @@ class ParserClient(BaseClient):
         return self._create_response(*http.request(url, 'POST', body=params))
 
     def _generate_url(self, resource, query_params=None):
-        """Build the url to resource.
+        """
+        Build the url to resource.
 
         :param resource: Name of the resource that is being called. Options are
         `''` (empty string) for root resource, `'parser'`, `'confidence'`.
         :param query_params: Data to be passed as query parameters.
-
         """
         if query_params:
             # extra & is for the token to be added
@@ -351,14 +400,15 @@ class ParserClient(BaseClient):
         return self.base_url_template.format(resource)
 
     def get_root(self):
-        """Send a GET request to the root resource of the Parser API.
-
+        """
+        Send a GET request to the root resource of the Parser API.
         """
         url = self._generate_url('')
         return self.get(url)
 
     def get_article_content(self, url=None, article_id=None, max_pages=25):
-        """Send a GET request to the `parser` endpoint of the parser API to get
+        """
+        Send a GET request to the `parser` endpoint of the parser API to get
         article content.
 
         The article can be identified by either a URL or an id that exists
@@ -382,7 +432,8 @@ class ParserClient(BaseClient):
         return self.get(url)
 
     def post_article_content(self, content, url, max_pages=25):
-        """POST content to be parsed to the Parser API.
+        """
+        POST content to be parsed to the Parser API.
 
         Note: Even when POSTing content, a url must still be provided.
 
@@ -390,7 +441,6 @@ class ParserClient(BaseClient):
         :param url: the url that represents the content
         :param max_pages (optional): the maximum number of pages to parse
             and combine. Default is 25.
-
         """
         params = {
             'doc': content,
@@ -401,7 +451,8 @@ class ParserClient(BaseClient):
         return self.post(url, post_params=params)
 
     def get_article_status(self, url=None, article_id=None):
-        """Send a HEAD request to the `parser` endpoint to the parser API to
+        """
+        Send a HEAD request to the `parser` endpoint to the parser API to
         get the articles status.
 
         Returned is a `httplib2.Response` object. The id and status for the
@@ -423,7 +474,8 @@ class ParserClient(BaseClient):
         return self.head(url)
 
     def get_confidence(self, url=None, article_id=None):
-        """Send a GET request to the `confidence` endpoint of the Parser API.
+        """
+        Send a GET request to the `confidence` endpoint of the Parser API.
 
         Note that either the `url` or `article_id` param should be passed.
 
